local table = require("hp/lang/table")
local class = require("hp/lang/class")

--------------------------------------------------------------------------------
-- ランダムダンジョンのマップを生成機です.
-- 任意のデータを与える事で、よくあるランダムダンジョンのマップデータの基礎を生成します.
-- @class table
-- @name DungeonBuilder
--------------------------------------------------------------------------------
local M = class()

--------------------------------------------------------------------------------
-- コンストラクタです.
--------------------------------------------------------------------------------
function M:init(params)
    self.width = params.width or 1
    self.height = params.height or 1
    self.ranges = {}
    self.rooms = {}
    self.roads = {}
    self.objects = {}
end

--------------------------------------------------------------------------------
-- 指定した座標のオブジェクトを返します.
-- 存在しなかった場合はnilを返します.
--------------------------------------------------------------------------------
function M:findObjectAt(x, y)
    for i, obj in ipairs(self.objects) do
        if obj.x == x and obj.y == y then
            return obj
        end
    end
end

--------------------------------------------------------------------------------
-- 領域を追加します.
--------------------------------------------------------------------------------
function M:addRange(range)
    return table.insertElement(self.ranges, range)
end

--------------------------------------------------------------------------------
-- 領域を削除します.
--------------------------------------------------------------------------------
function M:removeRange(range)
    return table.removeElement(self.ranges, range)
end


--------------------------------------------------------------------------------
-- 部屋を追加します.
--------------------------------------------------------------------------------
function M:addRoom(room)
    return table.insertElement(self.rooms, room)
end

--------------------------------------------------------------------------------
-- 部屋を削除します.
--------------------------------------------------------------------------------
function M:removeRoom(room)
    return table.removeElement(self.rooms, room)
end

--------------------------------------------------------------------------------
-- 道を追加します.
--------------------------------------------------------------------------------
function M:addRoad(road)
    return table.insertElement(self.roads, road)
end

--------------------------------------------------------------------------------
-- 道を削除します.
--------------------------------------------------------------------------------
function M:removeRoad(road)
    return table.removeElement(self.roads, road)
end

--------------------------------------------------------------------------------
-- オブジェクトを追加します.
--------------------------------------------------------------------------------
function M:addObject(object)
    return table.insertElement(self.objects, object)
end

--------------------------------------------------------------------------------
-- オブジェクトを削除します.
--------------------------------------------------------------------------------
function M:removeObject(object)
    return table.removeElement(self.objects, object)
end

return M